"""
Global configuration file for constant global immutable types.
"""

GLOBAL_TOKEN_FMT   = "$({name})"
GLOBAL_EXPAND_FMT  = "$(exp:{name})"
GLOBAL_REPEAT_GMT  = "$(rep:{name})"

FRUIT_FILENAME = "fruit.yaml"

ICON_SUCCESS = "✅" # ICON for showing good execution
ICON_ERROR   = "❌" # ICON for showing execution error
ICON_QUESTION = "❔"

ICON_FRUIT   = "🥝"


LINE_SEP = "-"*80
JOB_SEP  = ">"*80
SHELLCHAR = "$ "